@@ -6,6 +6,7 @@ Leip
 
 import argparse
 from collections import defaultdict
+import hashlib
 import logging
 import importlib
 import os
@@ -20,16 +21,21 @@ lg = logging.getLogger(__name__)
 #lg.setLevel(logging.DEBUG)
 
 
+#cache config files
+CONFIG = {}
+
+
 def get_config(name, config_files=None, base_location=None, base_config=None):
 
+    global CONFIG
+
     if base_config is None:
         if base_location is None:
             base_location = os.path.join('etc', '{}.config'.format(name))
         try:
             base_config = pkg_resources.resource_string(name, base_location)
         except IOError:
-
-            lg.critical("Leip cannot find package base config")
+            lg.debug("Leip cannot find package base config")
 
     if not config_files is None:
         config_files = [
@@ -37,7 +43,20 @@ def get_config(name, config_files=None, base_location=None, base_config=None):
             '/etc/{0}.config'.format(name),
             '~/.config/{0}/{0}.config'.format(name)]
 
-    return Yaco.PolyYaco(name, base=base_config, files=config_files)
+    md5 = hashlib.md5()
+    md5.update(name)
+    if not base_config is None:
+        md5.update(base_config)
+    md5.update(str(config_files))
+    config_digest = md5.hexdigest()
+
+    if CONFIG.has_key(config_digest):
+        lg.debug("returning digest from CONFIG cache ({}) ({})".format(config_digest, name))
+        return CONFIG[config_digest]
+
+    new_config = Yaco.PolyYaco(name, base=base_config, files=config_files)
+    CONFIG[config_digest] = new_config
+    return new_config
 
 
 class app(object):
@@ -133,7 +152,6 @@ class app(object):
 
                 lg.debug("attempting to load plugin from module {0}".format(
                     module_name))
-                modbase, modsub = module_name.rsplit('.', 1)
                 mod = importlib.import_module(module_name)
 
                 self.plugins[plugin_name] = mod
@@ -176,10 +194,27 @@ class app(object):
         else:
             mod_objects = mod.__dict__
 
+        self.run_init_hook(mod_objects)
+        self.discover_2(mod_objects)
+
+    def run_init_hook(self, mod_objects):
+        """
+        Run prediscovery initialization hook for this module.
+
+        This might allow, for example, flexible creation of functions
+        to be discovered later on.
+
+        For a hook to be executed as a prediscovery init hook, it needs to be
+        decorated with: ''@leip.init''
+        """
+
         leip_init_hook = None
         for obj_name in mod_objects:
             obj = mod_objects[obj_name]
 
+            if isinstance(obj, Yaco.Yaco):
+                continue
+
             #see if this is a function decorated as hook
             if hasattr(obj, '__call__') and \
                     hasattr(obj, '_leip_init_hook'):
@@ -190,16 +225,25 @@ class app(object):
             # the module has access to configuration
             leip_init_hook(self)
 
+    def discover_2(self, mod_objects):
+        """
+        Execute actual discovery of leip tagged functions & hooks
+        """
         for obj_name in mod_objects:
             obj = mod_objects[obj_name]
 
+            if isinstance(obj, Yaco.Yaco):
+                continue
+
             #see if this is a function decorated as hook
             if not hasattr(obj, '__call__'):
                 continue
 
             if hasattr(obj, '_leip_hook'):
                 hook = obj._leip_hook
-                prio = obj._leip_hook_priority
+                if isinstance(hook, Yaco.Yaco):
+                    continue
+                prio = obj.__dict__.get('_leip_hook_priority', 100)
                 lg.debug("discovered hook %s (%d) in %s" % (
                         hook, prio, obj.__name__))
                 self.hooks[hook].append(
@@ -208,7 +252,6 @@ class app(object):
             if hasattr(obj, '_leip_command'):
                 self.register_command(obj)
 
-
     def register_command(self, function):
         cname = function._leip_command
         lg.debug("discovered command %s" % cname)
@@ -221,21 +264,20 @@ class app(object):
             _desc = function.__doc__.strip().split("\n", 1)
 
         if len(_desc) == 2:
-            shortDesc, longDesc = _desc
+            short_description, long_description = _desc
         else:
-            shortDesc, longDesc = _desc[0], ""
+            short_description, long_description = _desc[0], ""
 
-        longDesc = textwrap.dedent(longDesc)
+        long_description = textwrap.dedent(long_description)
 
-        cp = self.subparser.add_parser(cname, help=shortDesc,
-                                       description=longDesc)
+        cp = self.subparser.add_parser(cname, help=short_description,
+                                       description=long_description)
 
         for args, kwargs in function._leip_args:
             cp.add_argument(*args, **kwargs)
 
         function._cparser = cp
 
-
     def register_hook(self, name, priority, function):
         lg.debug("registering hook {0} / {1}".format(name, function))
         self.hooks[name].append(
@@ -257,13 +299,9 @@ class app(object):
             self.run_hook(hook)
 
 
-
-
-
 ##
 ## Command decorators
 ##
-
 def command(f):
     """
     Tag a function to become a command - take the function name and
@@ -274,6 +312,7 @@ def command(f):
     lg.debug("marking function as leip command: %s" % f.__name__)
     return f
 
+
 def commandName(name):
     """
     as command, but provide a specific name
@@ -285,6 +324,7 @@ def commandName(name):
         return f
     return decorator
 
+
 def arg(*args, **kwargs):
     """
     add an argument to a command - use the full argparse syntax
@@ -295,6 +335,7 @@ def arg(*args, **kwargs):
         return f
     return decorator
 
+
 def flag(self, *args, **kwargs):
     """
     Add a flag to (default false - true if specified) any command
@@ -306,28 +347,24 @@ def flag(self, *args, **kwargs):
         return f
     return decorator
 
-##
-## Init hook decorators
-##
 
+##
+## Pre discovery init hook decorators
+##
 def init(f):
     """
-    mark this function as a hook for later execution
+    Mark this function as a pre discovery init hook.get_config.
 
-    :param name: name of the hook to call
-    :type name: string
-    :param priority: inidicate how soon this hook must be called.
-        Lower means sooner (default: 50)
-    :type priority: int
+    Only one per module is expected.
     """
     f._leip_init_hook = f.__name__
     return f
 
+
 ##
 ## Hook decorators
 ##
-
-def hook(name, priority = 50):
+def hook(name, priority=50):
     """
     mark this function as a hook for later execution
 
